@@ -225,13 +225,29 @@ class Precursors(ABC):
                 span_rowid=span_rowid,
             )
 
+    @dataclass(frozen=True)
+    class SessionAnnotation:
+        updated_at: datetime
+        session_id: str
+        obj: models.ProjectSessionAnnotation
+
+        def as_insertable(
+            self,
+            project_session_rowid: int,
+        ) -> Insertables.SessionAnnotation:
+            return Insertables.SessionAnnotation(
+                updated_at=self.updated_at,
+                session_id=self.session_id,
+                obj=self.obj,
+                project_session_rowid=project_session_rowid,
+            )
+
 
 class Insertables(ABC):
     @dataclass(frozen=True)
     class SpanAnnotation(Precursors.SpanAnnotation):
         updated_at: datetime
         span_rowid: int
-        identifier: str = ""
 
         @property
         def row(self) -> models.SpanAnnotation:
@@ -244,7 +260,6 @@ class Insertables(ABC):
     class TraceAnnotation(Precursors.TraceAnnotation):
         updated_at: datetime
         trace_rowid: int
-        identifier: str = ""
 
         @property
         def row(self) -> models.TraceAnnotation:
@@ -257,7 +272,6 @@ class Insertables(ABC):
     class DocumentAnnotation(Precursors.DocumentAnnotation):
         updated_at: datetime
         span_rowid: int
-        identifier: str = ""
 
         @property
         def row(self) -> models.DocumentAnnotation:
@@ -265,3 +279,15 @@ class Insertables(ABC):
             obj.span_rowid = self.span_rowid
             obj.updated_at = self.updated_at
             return obj
+
+    @dataclass(frozen=True)
+    class SessionAnnotation(Precursors.SessionAnnotation):
+        updated_at: datetime
+        project_session_rowid: int
+
+        @property
+        def row(self) -> models.ProjectSessionAnnotation:
+            obj = copy(self.obj)
+            obj.project_session_id = self.project_session_rowid
+            obj.updated_at = self.updated_at
+            return obj
