@@ -1,3 +1,4 @@
+import logging
 import secrets
 from contextlib import AsyncExitStack
 from datetime import datetime, timezone
@@ -8,6 +9,7 @@ from sqlalchemy import Boolean, Select, and_, case, cast, delete, distinct, func
 from sqlalchemy.exc import IntegrityError as PostgreSQLIntegrityError
 from sqlalchemy.orm import joinedload
 from sqlean.dbapi2 import IntegrityError as SQLiteIntegrityError  # type: ignore[import-untyped]
+from starlette.datastructures import Secret
 from strawberry import UNSET
 from strawberry.relay import GlobalID
 from strawberry.types import Info
@@ -19,26 +21,42 @@ from phoenix.auth import (
     PASSWORD_REQUIREMENTS,
     PHOENIX_ACCESS_TOKEN_COOKIE_NAME,
     PHOENIX_REFRESH_TOKEN_COOKIE_NAME,
+    sanitize_email,
     validate_email_format,
     validate_password_format,
 )
-from phoenix.db import enums, models
-from phoenix.server.api.auth import IsAdmin, IsLocked, IsNotReadOnly
+from phoenix.config import get_env_disable_basic_auth
+from phoenix.db import models
+from phoenix.server.api.auth import IsAdmin, IsLocked, IsNotReadOnly, IsNotViewer
 from phoenix.server.api.context import Context
-from phoenix.server.api.exceptions import Conflict, NotFound, Unauthorized
+from phoenix.server.api.exceptions import BadRequest, Conflict, NotFound, Unauthorized
 from phoenix.server.api.input_types.UserRoleInput import UserRoleInput
+from phoenix.server.api.types.AuthMethod import AuthMethod
 from phoenix.server.api.types.node import from_global_id_with_expected_type
 from phoenix.server.api.types.User import User, to_gql_user
 from phoenix.server.bearer_auth import PhoenixUser
 from phoenix.server.types import AccessTokenId, ApiKeyId, PasswordResetTokenId, RefreshTokenId
 
+logger = logging.getLogger(__name__)
+
 
 @strawberry.input
 class CreateUserInput:
     email: str
     username: str
-    password: str
+    password: Optional[str] = UNSET
     role: UserRoleInput
+    send_welcome_email: Optional[bool] = False
+    auth_method: Optional[AuthMethod] = AuthMethod.LOCAL
+
+    def __post_init__(self) -> None:
+        if self.auth_method is AuthMethod.OAUTH2:
+            if self.password:
+                raise BadRequest("Password is not allowed for OAuth2 authentication")
+        elif get_env_disable_basic_auth():
+            raise BadRequest("Basic auth is disabled: OAuth2 authentication only")
+        elif not self.password:
+            raise BadRequest("Password is required for local authentication")
 
 
 @strawberry.input
@@ -49,11 +67,16 @@ class PatchViewerInput:
 
     def __post_init__(self) -> None:
         if not self.new_username and not self.new_password:
-            raise ValueError("At least one field must be set")
-        if self.new_password and not self.current_password:
-            raise ValueError("current_password is required when modifying password")
+            raise BadRequest("At least one field must be set")
         if self.new_password:
-            PASSWORD_REQUIREMENTS.validate(self.new_password)
+            if get_env_disable_basic_auth():
+                raise BadRequest("Basic auth is disabled: OAuth2 authentication only")
+            if not self.current_password:
+                raise BadRequest("current_password is required when modifying password")
+            try:
+                PASSWORD_REQUIREMENTS.validate(self.new_password)
+            except ValueError as e:
+                raise BadRequest(str(e))
 
 
 @strawberry.input
@@ -65,9 +88,14 @@ class PatchUserInput:
 
     def __post_init__(self) -> None:
         if not self.new_role and not self.new_username and not self.new_password:
-            raise ValueError("At least one field must be set")
+            raise BadRequest("At least one field must be set")
         if self.new_password:
-            PASSWORD_REQUIREMENTS.validate(self.new_password)
+            if get_env_disable_basic_auth():
+                raise BadRequest("Basic auth is disabled: OAuth2 authentication only")
+            try:
+                PASSWORD_REQUIREMENTS.validate(self.new_password)
+            except ValueError as e:
+                raise BadRequest(str(e))
 
 
 @strawberry.input
@@ -82,23 +110,33 @@ class UserMutationPayload:
 
 @strawberry.type
 class UserMutationMixin:
-    @strawberry.mutation(permission_classes=[IsNotReadOnly, IsAdmin, IsLocked])  # type: ignore
+    @strawberry.mutation(permission_classes=[IsNotReadOnly, IsNotViewer, IsAdmin, IsLocked])  # type: ignore
     async def create_user(
         self,
         info: Info[Context, None],
         input: CreateUserInput,
     ) -> UserMutationPayload:
-        validate_email_format(email := input.email)
-        validate_password_format(password := input.password)
-        salt = secrets.token_bytes(DEFAULT_SECRET_LENGTH)
-        password_hash = await info.context.hash_password(password, salt)
-        user = models.User(
-            reset_password=True,
-            username=input.username,
-            email=email,
-            password_hash=password_hash,
-            password_salt=salt,
-        )
+        # Sanitize email by trimming and lowercasing
+        email = sanitize_email(input.email)
+
+        user: models.User
+        if input.auth_method is AuthMethod.OAUTH2:
+            user = models.OAuth2User(
+                email=email,
+                username=input.username,
+            )
+        else:
+            assert input.password
+            validate_email_format(email)
+            validate_password_format(input.password)
+            salt = secrets.token_bytes(DEFAULT_SECRET_LENGTH)
+            password_hash = await info.context.hash_password(Secret(input.password), salt)
+            user = models.LocalUser(
+                email=email,
+                username=input.username,
+                password_hash=password_hash,
+                password_salt=salt,
+            )
         async with AsyncExitStack() as stack:
             session = await stack.enter_async_context(info.context.db())
             user_role_id = await session.scalar(_select_role_id_by_name(input.role.value))
@@ -111,9 +149,15 @@ class UserMutationMixin:
                 await session.flush()
             except (PostgreSQLIntegrityError, SQLiteIntegrityError) as error:
                 raise Conflict(_user_operation_error_message(error))
+        if input.send_welcome_email and info.context.email_sender is not None:
+            try:
+                await info.context.email_sender.send_welcome_email(user.email, user.username)
+            except Exception as error:
+                # Log the error but do not raise it
+                logger.error(f"Failed to send welcome email: {error}")
         return UserMutationPayload(user=to_gql_user(user))
 
-    @strawberry.mutation(permission_classes=[IsNotReadOnly, IsAdmin, IsLocked])  # type: ignore
+    @strawberry.mutation(permission_classes=[IsNotReadOnly, IsNotViewer, IsAdmin])  # type: ignore
     async def patch_user(
         self,
         info: Info[Context, None],
@@ -130,20 +174,26 @@ class UserMutationMixin:
             if not (user := await session.scalar(_select_user_by_id(user_id))):
                 raise NotFound("User not found")
             stack.enter_context(session.no_autoflush)
+            should_log_out = False
             if input.new_role:
                 if user.email == DEFAULT_ADMIN_EMAIL:
                     raise Unauthorized("Cannot modify role for the default admin user")
+                if user_id == requester_id:
+                    raise Unauthorized("Cannot modify own role")
                 user_role_id = await session.scalar(_select_role_id_by_name(input.new_role.value))
                 if user_role_id is None:
                     raise NotFound(f"Role {input.new_role.value} not found")
                 user.user_role_id = user_role_id
+                should_log_out = True
             if password := input.new_password:
-                if user.auth_method != enums.AuthMethod.LOCAL.value:
+                if user.auth_method != "LOCAL":
                     raise Conflict("Cannot modify password for non-local user")
                 validate_password_format(password)
-                user.password_salt = secrets.token_bytes(DEFAULT_SECRET_LENGTH)
-                user.password_hash = await info.context.hash_password(password, user.password_salt)
+                salt = secrets.token_bytes(DEFAULT_SECRET_LENGTH)
+                user.password_salt = salt
+                user.password_hash = await info.context.hash_password(Secret(password), salt)
                 user.reset_password = True
+                should_log_out = True
             if username := input.new_username:
                 user.username = username
             assert user in session.dirty
@@ -152,11 +202,11 @@ class UserMutationMixin:
             except (PostgreSQLIntegrityError, SQLiteIntegrityError) as error:
                 raise Conflict(_user_operation_error_message(error, "modify"))
         assert user
-        if input.new_password:
+        if should_log_out:
             await info.context.log_out(user.id)
         return UserMutationPayload(user=to_gql_user(user))
 
-    @strawberry.mutation(permission_classes=[IsNotReadOnly, IsLocked])  # type: ignore
+    @strawberry.mutation(permission_classes=[IsNotReadOnly])  # type: ignore
     async def patch_viewer(
         self,
         info: Info[Context, None],
@@ -171,15 +221,16 @@ class UserMutationMixin:
                 raise NotFound("User not found")
             stack.enter_context(session.no_autoflush)
             if password := input.new_password:
-                if user.auth_method != enums.AuthMethod.LOCAL.value:
+                if user.auth_method != "LOCAL":
                     raise Conflict("Cannot modify password for non-local user")
                 if not (
                     current_password := input.current_password
-                ) or not await info.context.is_valid_password(current_password, user):
+                ) or not await info.context.is_valid_password(Secret(current_password), user):
                     raise Conflict("Valid current password is required to modify password")
                 validate_password_format(password)
-                user.password_salt = secrets.token_bytes(DEFAULT_SECRET_LENGTH)
-                user.password_hash = await info.context.hash_password(password, user.password_salt)
+                salt = secrets.token_bytes(DEFAULT_SECRET_LENGTH)
+                user.password_salt = salt
+                user.password_hash = await info.context.hash_password(Secret(password), salt)
                 user.reset_password = False
             if username := input.new_username:
                 user.username = username
@@ -197,7 +248,7 @@ class UserMutationMixin:
             response.delete_cookie(PHOENIX_ACCESS_TOKEN_COOKIE_NAME)
         return UserMutationPayload(user=to_gql_user(user))
 
-    @strawberry.mutation(permission_classes=[IsNotReadOnly, IsAdmin, IsLocked])  # type: ignore
+    @strawberry.mutation(permission_classes=[IsNotReadOnly, IsNotViewer, IsAdmin, IsLocked])  # type: ignore
     async def delete_users(
         self,
         info: Info[Context, None],
@@ -212,16 +263,8 @@ class UserMutationMixin:
                 set(input.user_ids),
             )
         )
-        system_user_role_id = (
-            select(models.UserRole.id)
-            .where(models.UserRole.name == enums.UserRole.SYSTEM.value)
-            .scalar_subquery()
-        )
-        admin_user_role_id = (
-            select(models.UserRole.id)
-            .where(models.UserRole.name == enums.UserRole.ADMIN.value)
-            .scalar_subquery()
-        )
+        system_user_role_id = select(models.UserRole.id).filter_by(name="SYSTEM").scalar_subquery()
+        admin_user_role_id = select(models.UserRole.id).filter_by(name="ADMIN").scalar_subquery()
         default_admin_user_id = (
             select(models.User.id)
             .where(
