@@ -3,6 +3,7 @@ import gzip
 import io
 import json
 import logging
+import urllib
 import zlib
 from asyncio import QueueFull
 from collections import Counter
@@ -14,22 +15,14 @@ from typing import Any, Optional, Union, cast
 
 import pandas as pd
 import pyarrow as pa
-from fastapi import APIRouter, BackgroundTasks, HTTPException, Path, Query
+from fastapi import APIRouter, BackgroundTasks, Depends, HTTPException, Path, Query
 from fastapi.responses import PlainTextResponse, StreamingResponse
-from sqlalchemy import and_, delete, func, select
+from sqlalchemy import and_, case, delete, func, select
 from sqlalchemy.ext.asyncio import AsyncSession
 from starlette.concurrency import run_in_threadpool
 from starlette.datastructures import FormData, UploadFile
 from starlette.requests import Request
 from starlette.responses import Response
-from starlette.status import (
-    HTTP_200_OK,
-    HTTP_204_NO_CONTENT,
-    HTTP_404_NOT_FOUND,
-    HTTP_409_CONFLICT,
-    HTTP_422_UNPROCESSABLE_ENTITY,
-    HTTP_429_TOO_MANY_REQUESTS,
-)
 from strawberry.relay import GlobalID
 from typing_extensions import TypeAlias, assert_never
 
@@ -46,6 +39,8 @@ from phoenix.server.api.types.DatasetExample import DatasetExample as DatasetExa
 from phoenix.server.api.types.DatasetVersion import DatasetVersion as DatasetVersionNodeType
 from phoenix.server.api.types.node import from_global_id_with_expected_type
 from phoenix.server.api.utils import delete_projects, delete_traces
+from phoenix.server.authorization import is_not_locked
+from phoenix.server.bearer_auth import PhoenixUser
 from phoenix.server.dml_event import DatasetInsertEvent
 
 from .models import V1RoutesBaseModel
@@ -56,6 +51,11 @@ from .utils import (
     add_text_csv_content_to_responses,
 )
 
+csv.field_size_limit(
+    1_000_000_000  # allows large field sizes for CSV upload (1GB)
+)
+
+
 logger = logging.getLogger(__name__)
 
 DATASET_NODE_NAME = DatasetNodeType.__name__
@@ -72,6 +72,7 @@ class Dataset(V1RoutesBaseModel):
     metadata: dict[str, Any]
     created_at: datetime
     updated_at: datetime
+    example_count: int
 
 
 class ListDatasetsResponseBody(PaginatedResponseBody[Dataset]):
@@ -82,7 +83,7 @@ class ListDatasetsResponseBody(PaginatedResponseBody[Dataset]):
     "/datasets",
     operation_id="listDatasets",
     summary="List datasets",
-    responses=add_errors_to_responses([HTTP_422_UNPROCESSABLE_ENTITY]),
+    responses=add_errors_to_responses([422]),
 )
 async def list_datasets(
     request: Request,
@@ -96,7 +97,18 @@ async def list_datasets(
     ),
 ) -> ListDatasetsResponseBody:
     async with request.app.state.db() as session:
-        query = select(models.Dataset).order_by(models.Dataset.id.desc())
+        value = case(
+            (models.DatasetExampleRevision.revision_kind == "CREATE", 1),
+            (models.DatasetExampleRevision.revision_kind == "DELETE", -1),
+        )
+        query = (
+            select(models.Dataset)
+            .add_columns(func.coalesce(func.sum(value), 0).label("example_count"))
+            .outerjoin_from(models.Dataset, models.DatasetExample)
+            .outerjoin_from(models.DatasetExample, models.DatasetExampleRevision)
+            .group_by(models.Dataset.id)
+            .order_by(models.Dataset.id.desc())
+        )
 
         if cursor:
             try:
@@ -105,25 +117,26 @@ async def list_datasets(
             except ValueError:
                 raise HTTPException(
                     detail=f"Invalid cursor format: {cursor}",
-                    status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+                    status_code=422,
                 )
         if name:
             query = query.filter(models.Dataset.name == name)
 
         query = query.limit(limit + 1)
         result = await session.execute(query)
-        datasets = result.scalars().all()
-
+        datasets = result.all()
         if not datasets:
             return ListDatasetsResponseBody(next_cursor=None, data=[])
 
         next_cursor = None
         if len(datasets) == limit + 1:
-            next_cursor = str(GlobalID(DATASET_NODE_NAME, str(datasets[-1].id)))
+            dataset = datasets[-1][0]
+            next_cursor = str(GlobalID(DATASET_NODE_NAME, str(dataset.id)))
             datasets = datasets[:-1]
 
         data = []
-        for dataset in datasets:
+        for row in datasets:
+            dataset = row[0]
             data.append(
                 Dataset(
                     id=str(GlobalID(DATASET_NODE_NAME, str(dataset.id))),
@@ -132,6 +145,7 @@ async def list_datasets(
                     metadata=dataset.metadata_,
                     created_at=dataset.created_at,
                     updated_at=dataset.updated_at,
+                    example_count=row[1],
                 )
             )
 
@@ -142,11 +156,11 @@ async def list_datasets(
     "/datasets/{id}",
     operation_id="deleteDatasetById",
     summary="Delete dataset by ID",
-    status_code=HTTP_204_NO_CONTENT,
+    status_code=204,
     responses=add_errors_to_responses(
         [
-            {"status_code": HTTP_404_NOT_FOUND, "description": "Dataset not found"},
-            {"status_code": HTTP_422_UNPROCESSABLE_ENTITY, "description": "Invalid dataset ID"},
+            {"status_code": 404, "description": "Dataset not found"},
+            {"status_code": 422, "description": "Invalid dataset ID"},
         ]
     ),
 )
@@ -160,11 +174,9 @@ async def delete_dataset(
                 DATASET_NODE_NAME,
             )
         except ValueError:
-            raise HTTPException(
-                detail=f"Invalid Dataset ID: {id}", status_code=HTTP_422_UNPROCESSABLE_ENTITY
-            )
+            raise HTTPException(detail=f"Invalid Dataset ID: {id}", status_code=422)
     else:
-        raise HTTPException(detail="Missing Dataset ID", status_code=HTTP_422_UNPROCESSABLE_ENTITY)
+        raise HTTPException(detail="Missing Dataset ID", status_code=422)
     project_names_stmt = get_project_names_for_datasets(dataset_id)
     eval_trace_ids_stmt = get_eval_trace_ids_for_datasets(dataset_id)
     stmt = (
@@ -174,7 +186,7 @@ async def delete_dataset(
         project_names = await session.scalars(project_names_stmt)
         eval_trace_ids = await session.scalars(eval_trace_ids_stmt)
         if (await session.scalar(stmt)) is None:
-            raise HTTPException(detail="Dataset does not exist", status_code=HTTP_404_NOT_FOUND)
+            raise HTTPException(detail="Dataset does not exist", status_code=404)
     tasks = BackgroundTasks()
     tasks.add_task(delete_projects, request.app.state.db, *project_names)
     tasks.add_task(delete_traces, request.app.state.db, *eval_trace_ids)
@@ -192,17 +204,21 @@ class GetDatasetResponseBody(ResponseBody[DatasetWithExampleCount]):
     "/datasets/{id}",
     operation_id="getDataset",
     summary="Get dataset by ID",
-    responses=add_errors_to_responses([HTTP_404_NOT_FOUND]),
+    responses=add_errors_to_responses([404]),
 )
 async def get_dataset(
     request: Request, id: str = Path(description="The ID of the dataset")
 ) -> GetDatasetResponseBody:
-    dataset_id = GlobalID.from_id(id)
+    try:
+        dataset_id = GlobalID.from_id(id)
+    except Exception as e:
+        raise HTTPException(
+            detail=f"Invalid dataset ID format: {id}",
+            status_code=422,
+        ) from e
 
     if (type_name := dataset_id.type_name) != DATASET_NODE_NAME:
-        raise HTTPException(
-            detail=f"ID {dataset_id} refers to a f{type_name}", status_code=HTTP_404_NOT_FOUND
-        )
+        raise HTTPException(detail=f"ID {dataset_id} refers to a f{type_name}", status_code=404)
     async with request.app.state.db() as session:
         result = await session.execute(
             select(models.Dataset, models.Dataset.example_count).filter(
@@ -213,9 +229,7 @@ async def get_dataset(
         dataset = dataset_query[0] if dataset_query else None
         example_count = dataset_query[1] if dataset_query else 0
         if dataset is None:
-            raise HTTPException(
-                detail=f"Dataset with ID {dataset_id} not found", status_code=HTTP_404_NOT_FOUND
-            )
+            raise HTTPException(detail=f"Dataset with ID {dataset_id} not found", status_code=404)
 
         dataset = DatasetWithExampleCount(
             id=str(dataset_id),
@@ -244,7 +258,7 @@ class ListDatasetVersionsResponseBody(PaginatedResponseBody[DatasetVersion]):
     "/datasets/{id}/versions",
     operation_id="listDatasetVersionsByDatasetId",
     summary="List dataset versions",
-    responses=add_errors_to_responses([HTTP_422_UNPROCESSABLE_ENTITY]),
+    responses=add_errors_to_responses([422]),
 )
 async def list_dataset_versions(
     request: Request,
@@ -266,12 +280,12 @@ async def list_dataset_versions(
         except ValueError:
             raise HTTPException(
                 detail=f"Invalid Dataset ID: {id}",
-                status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+                status_code=422,
             )
     else:
         raise HTTPException(
             detail="Missing Dataset ID",
-            status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+            status_code=422,
         )
     stmt = (
         select(models.DatasetVersion)
@@ -287,7 +301,7 @@ async def list_dataset_versions(
         except ValueError:
             raise HTTPException(
                 detail=f"Invalid cursor: {cursor}",
-                status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+                status_code=422,
             )
         max_dataset_version_id = (
             select(models.DatasetVersion.id)
@@ -311,6 +325,7 @@ async def list_dataset_versions(
 
 class UploadDatasetData(V1RoutesBaseModel):
     dataset_id: str
+    version_id: str
 
 
 class UploadDatasetResponseBody(ResponseBody[UploadDatasetData]):
@@ -319,15 +334,16 @@ class UploadDatasetResponseBody(ResponseBody[UploadDatasetData]):
 
 @router.post(
     "/datasets/upload",
+    dependencies=[Depends(is_not_locked)],
     operation_id="uploadDataset",
     summary="Upload dataset from JSON, CSV, or PyArrow",
     responses=add_errors_to_responses(
         [
             {
-                "status_code": HTTP_409_CONFLICT,
+                "status_code": 409,
                 "description": "Dataset of the same name already exists",
             },
-            {"status_code": HTTP_422_UNPROCESSABLE_ENTITY, "description": "Invalid request body"},
+            {"status_code": 422, "description": "Invalid request body"},
         ]
     ),
     # FastAPI cannot generate the request body portion of the OpenAPI schema for
@@ -390,7 +406,12 @@ async def upload_dataset(
         description="If true, fulfill request synchronously and return JSON containing dataset_id.",
     ),
 ) -> Optional[UploadDatasetResponseBody]:
-    request_content_type = request.headers["content-type"]
+    request_content_type = request.headers.get("content-type")
+    if not request_content_type:
+        raise HTTPException(
+            detail="Missing content-type header",
+            status_code=400,
+        )
     examples: Union[Examples, Awaitable[Examples]]
     if request_content_type.startswith("application/json"):
         try:
@@ -400,14 +421,14 @@ async def upload_dataset(
         except ValueError as e:
             raise HTTPException(
                 detail=str(e),
-                status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+                status_code=422,
             )
         if action is DatasetAction.CREATE:
             async with request.app.state.db() as session:
                 if await _check_table_exists(session, name):
                     raise HTTPException(
                         detail=f"Dataset with the same name already exists: {name=}",
-                        status_code=HTTP_409_CONFLICT,
+                        status_code=409,
                     )
     elif request_content_type.startswith("multipart/form-data"):
         async with request.form() as form:
@@ -424,14 +445,14 @@ async def upload_dataset(
             except ValueError as e:
                 raise HTTPException(
                     detail=str(e),
-                    status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+                    status_code=422,
                 )
             if action is DatasetAction.CREATE:
                 async with request.app.state.db() as session:
                     if await _check_table_exists(session, name):
                         raise HTTPException(
                             detail=f"Dataset with the same name already exists: {name=}",
-                            status_code=HTTP_409_CONFLICT,
+                            status_code=409,
                         )
             content = await file.read()
         try:
@@ -448,13 +469,16 @@ async def upload_dataset(
         except ValueError as e:
             raise HTTPException(
                 detail=str(e),
-                status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+                status_code=422,
             )
     else:
         raise HTTPException(
             detail="Invalid request Content-Type",
-            status_code=HTTP_422_UNPROCESSABLE_ENTITY,
+            status_code=422,
         )
+    user_id: Optional[int] = None
+    if request.app.state.authentication_enabled and isinstance(request.user, PhoenixUser):
+        user_id = int(request.user.identity)
     operation = cast(
         Callable[[AsyncSession], Awaitable[DatasetExampleAdditionEvent]],
         partial(
@@ -463,21 +487,27 @@ async def upload_dataset(
             action=action,
             name=name,
             description=description,
+            user_id=user_id,
         ),
     )
     if sync:
         async with request.app.state.db() as session:
-            dataset_id = (await operation(session)).dataset_id
+            event = await operation(session)
+            dataset_id = event.dataset_id
+            version_id = event.dataset_version_id
         request.state.event_queue.put(DatasetInsertEvent((dataset_id,)))
         return UploadDatasetResponseBody(
-            data=UploadDatasetData(dataset_id=str(GlobalID(Dataset.__name__, str(dataset_id))))
+            data=UploadDatasetData(
+                dataset_id=str(GlobalID(Dataset.__name__, str(dataset_id))),
+                version_id=str(GlobalID(DatasetVersion.__name__, str(version_id))),
+            )
         )
     try:
         request.state.enqueue_operation(operation)
     except QueueFull:
         if isinstance(examples, Coroutine):
             examples.close()
-        raise HTTPException(detail="Too many requests.", status_code=HTTP_429_TOO_MANY_REQUESTS)
+        raise HTTPException(detail="Too many requests.", status_code=429)
     return None
 
 
@@ -678,7 +708,7 @@ class ListDatasetExamplesResponseBody(ResponseBody[ListDatasetExamplesData]):
     "/datasets/{id}/examples",
     operation_id="getDatasetExamples",
     summary="Get examples from a dataset",
-    responses=add_errors_to_responses([HTTP_404_NOT_FOUND]),
+    responses=add_errors_to_responses([404]),
 )
 async def get_dataset_examples(
     request: Request,
@@ -686,22 +716,34 @@ async def get_dataset_examples(
     version_id: Optional[str] = Query(
         default=None,
         description=(
-            "The ID of the dataset version " "(if omitted, returns data from the latest version)"
+            "The ID of the dataset version (if omitted, returns data from the latest version)"
         ),
     ),
 ) -> ListDatasetExamplesResponseBody:
-    dataset_gid = GlobalID.from_id(id)
-    version_gid = GlobalID.from_id(version_id) if version_id else None
+    try:
+        dataset_gid = GlobalID.from_id(id)
+    except Exception as e:
+        raise HTTPException(
+            detail=f"Invalid dataset ID format: {id}",
+            status_code=422,
+        ) from e
+
+    if version_id:
+        try:
+            version_gid = GlobalID.from_id(version_id)
+        except Exception as e:
+            raise HTTPException(
+                detail=f"Invalid dataset version ID format: {version_id}",
+                status_code=422,
+            ) from e
+    else:
+        version_gid = None
 
     if (dataset_type := dataset_gid.type_name) != "Dataset":
-        raise HTTPException(
-            detail=f"ID {dataset_gid} refers to a {dataset_type}", status_code=HTTP_404_NOT_FOUND
-        )
+        raise HTTPException(detail=f"ID {dataset_gid} refers to a {dataset_type}", status_code=404)
 
     if version_gid and (version_type := version_gid.type_name) != "DatasetVersion":
-        raise HTTPException(
-            detail=f"ID {version_gid} refers to a {version_type}", status_code=HTTP_404_NOT_FOUND
-        )
+        raise HTTPException(detail=f"ID {version_gid} refers to a {version_type}", status_code=404)
 
     async with request.app.state.db() as session:
         if (
@@ -711,7 +753,7 @@ async def get_dataset_examples(
         ) is None:
             raise HTTPException(
                 detail=f"No dataset with id {dataset_gid} can be found.",
-                status_code=HTTP_404_NOT_FOUND,
+                status_code=404,
             )
 
         # Subquery to find the maximum created_at for each dataset_example_id
@@ -733,7 +775,7 @@ async def get_dataset_examples(
             ) is None:
                 raise HTTPException(
                     detail=f"No dataset version with id {version_id} can be found.",
-                    status_code=HTTP_404_NOT_FOUND,
+                    status_code=404,
                 )
             # if a version_id is provided, filter the subquery to only include revisions from that
             partial_subquery = partial_subquery.filter(
@@ -749,7 +791,7 @@ async def get_dataset_examples(
             ) is None:
                 raise HTTPException(
                     detail="Dataset has no versions.",
-                    status_code=HTTP_404_NOT_FOUND,
+                    status_code=404,
                 )
 
         subquery = partial_subquery.subquery()
@@ -792,10 +834,10 @@ async def get_dataset_examples(
     operation_id="getDatasetCsv",
     summary="Download dataset examples as CSV file",
     response_class=StreamingResponse,
-    status_code=HTTP_200_OK,
+    status_code=200,
     responses={
-        **add_errors_to_responses([HTTP_422_UNPROCESSABLE_ENTITY]),
-        **add_text_csv_content_to_responses(HTTP_200_OK),
+        **add_errors_to_responses([422]),
+        **add_text_csv_content_to_responses(200),
     },
 )
 async def get_dataset_csv(
@@ -805,7 +847,7 @@ async def get_dataset_csv(
     version_id: Optional[str] = Query(
         default=None,
         description=(
-            "The ID of the dataset version " "(if omitted, returns data from the latest version)"
+            "The ID of the dataset version (if omitted, returns data from the latest version)"
         ),
     ),
 ) -> Response:
@@ -815,12 +857,13 @@ async def get_dataset_csv(
                 session=session, id=id, version_id=version_id
             )
     except ValueError as e:
-        raise HTTPException(detail=str(e), status_code=HTTP_422_UNPROCESSABLE_ENTITY)
+        raise HTTPException(detail=str(e), status_code=422)
     content = await run_in_threadpool(_get_content_csv, examples)
+    encoded_dataset_name = urllib.parse.quote(dataset_name)
     return Response(
         content=content,
         headers={
-            "content-disposition": f'attachment; filename="{dataset_name}.csv"',
+            "content-disposition": f"attachment; filename*=UTF-8''{encoded_dataset_name}.csv",
             "content-type": "text/csv",
         },
     )
@@ -834,7 +877,7 @@ async def get_dataset_csv(
     responses=add_errors_to_responses(
         [
             {
-                "status_code": HTTP_422_UNPROCESSABLE_ENTITY,
+                "status_code": 422,
                 "description": "Invalid dataset or version ID",
             }
         ]
@@ -847,7 +890,7 @@ async def get_dataset_jsonl_openai_ft(
     version_id: Optional[str] = Query(
         default=None,
         description=(
-            "The ID of the dataset version " "(if omitted, returns data from the latest version)"
+            "The ID of the dataset version (if omitted, returns data from the latest version)"
         ),
     ),
 ) -> bytes:
@@ -857,9 +900,12 @@ async def get_dataset_jsonl_openai_ft(
                 session=session, id=id, version_id=version_id
             )
     except ValueError as e:
-        raise HTTPException(detail=str(e), status_code=HTTP_422_UNPROCESSABLE_ENTITY)
+        raise HTTPException(detail=str(e), status_code=422)
     content = await run_in_threadpool(_get_content_jsonl_openai_ft, examples)
-    response.headers["content-disposition"] = f'attachment; filename="{dataset_name}.jsonl"'
+    encoded_dataset_name = urllib.parse.quote(dataset_name)
+    response.headers["content-disposition"] = (
+        f"attachment; filename*=UTF-8''{encoded_dataset_name}.jsonl"
+    )
     return content
 
 
@@ -871,7 +917,7 @@ async def get_dataset_jsonl_openai_ft(
     responses=add_errors_to_responses(
         [
             {
-                "status_code": HTTP_422_UNPROCESSABLE_ENTITY,
+                "status_code": 422,
                 "description": "Invalid dataset or version ID",
             }
         ]
@@ -884,7 +930,7 @@ async def get_dataset_jsonl_openai_evals(
     version_id: Optional[str] = Query(
         default=None,
         description=(
-            "The ID of the dataset version " "(if omitted, returns data from the latest version)"
+            "The ID of the dataset version (if omitted, returns data from the latest version)"
         ),
     ),
 ) -> bytes:
@@ -894,9 +940,12 @@ async def get_dataset_jsonl_openai_evals(
                 session=session, id=id, version_id=version_id
             )
     except ValueError as e:
-        raise HTTPException(detail=str(e), status_code=HTTP_422_UNPROCESSABLE_ENTITY)
+        raise HTTPException(detail=str(e), status_code=422)
     content = await run_in_threadpool(_get_content_jsonl_openai_evals, examples)
-    response.headers["content-disposition"] = f'attachment; filename="{dataset_name}.jsonl"'
+    encoded_dataset_name = urllib.parse.quote(dataset_name)
+    response.headers["content-disposition"] = (
+        f"attachment; filename*=UTF-8''{encoded_dataset_name}.jsonl"
+    )
     return content
 
 
@@ -970,12 +1019,25 @@ def _get_content_jsonl_openai_evals(examples: list[models.DatasetExampleRevision
 async def _get_db_examples(
     *, session: Any, id: str, version_id: Optional[str]
 ) -> tuple[str, list[models.DatasetExampleRevision]]:
-    dataset_id = from_global_id_with_expected_type(GlobalID.from_id(id), DATASET_NODE_NAME)
+    try:
+        dataset_id = from_global_id_with_expected_type(GlobalID.from_id(id), DATASET_NODE_NAME)
+    except Exception as e:
+        raise HTTPException(
+            detail=f"Invalid dataset ID format: {id}",
+            status_code=422,
+        ) from e
+
     dataset_version_id: Optional[int] = None
     if version_id:
-        dataset_version_id = from_global_id_with_expected_type(
-            GlobalID.from_id(version_id), DATASET_VERSION_NODE_NAME
-        )
+        try:
+            dataset_version_id = from_global_id_with_expected_type(
+                GlobalID.from_id(version_id), DATASET_VERSION_NODE_NAME
+            )
+        except Exception as e:
+            raise HTTPException(
+                detail=f"Invalid dataset version ID format: {version_id}",
+                status_code=422,
+            ) from e
     latest_version = (
         select(
             models.DatasetExampleRevision.dataset_example_id,
