@@ -3,7 +3,7 @@ from datetime import datetime
 from typing import ClassVar, Optional, cast
 
 import strawberry
-from sqlalchemy import and_, func, or_, select
+from sqlalchemy import Text, and_, func, or_, select
 from sqlalchemy.sql.functions import count
 from strawberry import UNSET
 from strawberry.relay import Connection, GlobalID, Node, NodeID
@@ -18,6 +18,8 @@ from phoenix.server.api.types.DatasetExample import DatasetExample
 from phoenix.server.api.types.DatasetExperimentAnnotationSummary import (
     DatasetExperimentAnnotationSummary,
 )
+from phoenix.server.api.types.DatasetLabel import DatasetLabel, to_gql_dataset_label
+from phoenix.server.api.types.DatasetSplit import DatasetSplit, to_gql_dataset_split
 from phoenix.server.api.types.DatasetVersion import DatasetVersion
 from phoenix.server.api.types.Experiment import Experiment, to_gql_experiment
 from phoenix.server.api.types.node import from_global_id_with_expected_type
@@ -86,6 +88,7 @@ class Dataset(Node):
         self,
         info: Info[Context, None],
         dataset_version_id: Optional[GlobalID] = UNSET,
+        split_ids: Optional[list[GlobalID]] = UNSET,
     ) -> int:
         dataset_id = self.id_attr
         version_id = (
@@ -96,6 +99,20 @@ class Dataset(Node):
             if dataset_version_id
             else None
         )
+
+        # Parse split IDs if provided
+        split_rowids: Optional[list[int]] = None
+        if split_ids:
+            split_rowids = []
+            for split_id in split_ids:
+                try:
+                    split_rowid = from_global_id_with_expected_type(
+                        global_id=split_id, expected_type_name=models.DatasetSplit.__name__
+                    )
+                    split_rowids.append(split_rowid)
+                except Exception:
+                    raise BadRequest(f"Invalid split ID: {split_id}")
+
         revision_ids = (
             select(func.max(models.DatasetExampleRevision.id))
             .join(models.DatasetExample)
@@ -112,11 +129,36 @@ class Dataset(Node):
             revision_ids = revision_ids.where(
                 models.DatasetExampleRevision.dataset_version_id <= version_id_subquery
             )
-        stmt = (
-            select(count(models.DatasetExampleRevision.id))
-            .where(models.DatasetExampleRevision.id.in_(revision_ids))
-            .where(models.DatasetExampleRevision.revision_kind != "DELETE")
-        )
+
+        # Build the count query
+        if split_rowids:
+            # When filtering by splits, count distinct examples that belong to those splits
+            stmt = (
+                select(count(models.DatasetExample.id.distinct()))
+                .join(
+                    models.DatasetExampleRevision,
+                    onclause=(
+                        models.DatasetExample.id == models.DatasetExampleRevision.dataset_example_id
+                    ),
+                )
+                .join(
+                    models.DatasetSplitDatasetExample,
+                    onclause=(
+                        models.DatasetExample.id
+                        == models.DatasetSplitDatasetExample.dataset_example_id
+                    ),
+                )
+                .where(models.DatasetExampleRevision.id.in_(revision_ids))
+                .where(models.DatasetExampleRevision.revision_kind != "DELETE")
+                .where(models.DatasetSplitDatasetExample.dataset_split_id.in_(split_rowids))
+            )
+        else:
+            stmt = (
+                select(count(models.DatasetExampleRevision.id))
+                .where(models.DatasetExampleRevision.id.in_(revision_ids))
+                .where(models.DatasetExampleRevision.revision_kind != "DELETE")
+            )
+
         async with info.context.db() as session:
             return (await session.scalar(stmt)) or 0
 
@@ -125,10 +167,12 @@ class Dataset(Node):
         self,
         info: Info[Context, None],
         dataset_version_id: Optional[GlobalID] = UNSET,
+        split_ids: Optional[list[GlobalID]] = UNSET,
         first: Optional[int] = 50,
         last: Optional[int] = UNSET,
         after: Optional[CursorString] = UNSET,
         before: Optional[CursorString] = UNSET,
+        filter: Optional[str] = UNSET,
     ) -> Connection[DatasetExample]:
         args = ConnectionArgs(
             first=first,
@@ -144,6 +188,20 @@ class Dataset(Node):
             if dataset_version_id
             else None
         )
+
+        # Parse split IDs if provided
+        split_rowids: Optional[list[int]] = None
+        if split_ids:
+            split_rowids = []
+            for split_id in split_ids:
+                try:
+                    split_rowid = from_global_id_with_expected_type(
+                        global_id=split_id, expected_type_name=models.DatasetSplit.__name__
+                    )
+                    split_rowids.append(split_rowid)
+                except Exception:
+                    raise BadRequest(f"Invalid split ID: {split_id}")
+
         revision_ids = (
             select(func.max(models.DatasetExampleRevision.id))
             .join(models.DatasetExample)
@@ -175,6 +233,31 @@ class Dataset(Node):
             )
             .order_by(models.DatasetExampleRevision.dataset_example_id.desc())
         )
+
+        # Filter by split IDs if provided
+        if split_rowids:
+            query = (
+                query.join(
+                    models.DatasetSplitDatasetExample,
+                    onclause=(
+                        models.DatasetExample.id
+                        == models.DatasetSplitDatasetExample.dataset_example_id
+                    ),
+                )
+                .where(models.DatasetSplitDatasetExample.dataset_split_id.in_(split_rowids))
+                .distinct()
+            )
+        # Apply filter if provided - search through JSON fields (input, output, metadata)
+        if filter is not UNSET and filter:
+            # Create a filter that searches for the filter string in JSON fields
+            # Using PostgreSQL's JSON operators for case-insensitive text search
+            filter_condition = or_(
+                func.cast(models.DatasetExampleRevision.input, Text).ilike(f"%{filter}%"),
+                func.cast(models.DatasetExampleRevision.output, Text).ilike(f"%{filter}%"),
+                func.cast(models.DatasetExampleRevision.metadata_, Text).ilike(f"%{filter}%"),
+            )
+            query = query.where(filter_condition)
+
         async with info.context.db() as session:
             dataset_examples = [
                 DatasetExample(
@@ -186,6 +269,13 @@ class Dataset(Node):
             ]
         return connection_from_list(data=dataset_examples, args=args)
 
+    @strawberry.field
+    async def splits(self, info: Info[Context, None]) -> list[DatasetSplit]:
+        return [
+            to_gql_dataset_split(split)
+            for split in await info.context.data_loaders.dataset_dataset_splits.load(self.id_attr)
+        ]
+
     @strawberry.field(
         description="Number of experiments for a specific version if version is specified, "
         "or for all versions if version is not specified."
@@ -303,6 +393,13 @@ class Dataset(Node):
                 async for scores_tuple in await session.stream(query)
             ]
 
+    @strawberry.field
+    async def labels(self, info: Info[Context, None]) -> list[DatasetLabel]:
+        return [
+            to_gql_dataset_label(label)
+            for label in await info.context.data_loaders.dataset_labels.load(self.id_attr)
+        ]
+
     @strawberry.field
     def last_updated_at(self, info: Info[Context, None]) -> Optional[datetime]:
         return info.context.last_updated_at.get(self._table, self.id_attr)
