@@ -2,7 +2,6 @@ import csv
 import gzip
 import logging
 import re
-import weakref
 from collections import Counter
 from collections.abc import Iterable, Mapping, Sequence
 from datetime import datetime
@@ -20,7 +19,7 @@ from opentelemetry.proto.common.v1.common_pb2 import AnyValue, KeyValue
 from opentelemetry.proto.resource.v1.resource_pb2 import Resource
 from opentelemetry.proto.trace.v1.trace_pb2 import ResourceSpans, ScopeSpans
 from pyarrow import ArrowInvalid, Table
-from typing_extensions import TypeAlias, assert_never
+from typing_extensions import TypeAlias, assert_never, deprecated
 
 from phoenix.config import (
     get_env_collector_endpoint,
@@ -69,8 +68,7 @@ class Client(TraceDataExtractor):
         """
         if kwargs.pop("use_active_session_if_available", None) is not None:
             print(
-                "`use_active_session_if_available` is deprecated "
-                "and will be removed in the future."
+                "`use_active_session_if_available` is deprecated and will be removed in the future."
             )
         if kwargs:
             raise TypeError(f"Unexpected keyword arguments: {', '.join(kwargs)}")
@@ -84,11 +82,16 @@ class Client(TraceDataExtractor):
         if host == "0.0.0.0":
             host = "127.0.0.1"
         base_url = endpoint or get_env_collector_endpoint() or f"http://{host}:{get_env_port()}"
-        self._base_url = base_url if base_url.endswith("/") else base_url + "/"
-        self._client = VersionedClient(headers=headers)
-        weakref.finalize(self, self._client.close)
+        self._client = VersionedClient(headers=headers, base_url=httpx.URL(base_url))
         if warn_if_server_not_running:
             self._warn_if_phoenix_is_not_running()
+        try:
+            # place import here temporarily before phoenix-client is fully published on pypi
+            from phoenix.client.resources.prompts import Prompts
+
+            self.prompts = Prompts(self._client)
+        except ImportError:
+            pass
 
     @property
     def web_url(self) -> str:
@@ -105,8 +108,9 @@ class Client(TraceDataExtractor):
 
         if session := active_session():
             return session.url
-        return self._base_url
+        return str(self._client.base_url)
 
+    @deprecated("Migrate to using client.spans.get_spans_dataframe via arize-phoenix-client")
     def query_spans(
         self,
         *queries: SpanQuery,
@@ -118,8 +122,14 @@ class Client(TraceDataExtractor):
         # Deprecated
         stop_time: Optional[datetime] = None,
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
+        orphan_span_as_root_span: bool = True,
     ) -> Optional[Union[pd.DataFrame, list[pd.DataFrame]]]:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.spans.get_spans_dataframe()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Queries spans from the Phoenix server or active session based on specified criteria.
 
         Args:
@@ -127,9 +137,12 @@ class Client(TraceDataExtractor):
             start_time (datetime, optional): The start time for the query range. Default None.
             end_time (datetime, optional): The end time for the query range. Default None.
             root_spans_only (bool, optional): If True, only root spans are returned. Default None.
+            orphan_span_as_root_span (bool): If True, orphan spans are treated as root spans. An
+                orphan span has a non-null `parent_id` but a span with that ID is currently not
+                found in the database. Default True.
             project_name (str, optional): The project name to query spans for. This can be set
                 using environment variables. If not provided, falls back to the default project.
-           timeout (int, optional): The number of seconds to wait for the server to respond.
+            timeout (int, optional): The number of seconds to wait for the server to respond.
 
         Returns:
             Union[pd.DataFrame, list[pd.DataFrame]]:
@@ -147,8 +160,8 @@ class Client(TraceDataExtractor):
             end_time = end_time or stop_time
         try:
             response = self._client.post(
+                url="v1/spans",
                 headers={"accept": "application/json"},
-                url=urljoin(self._base_url, "v1/spans"),
                 params={
                     "project_name": project_name,
                     "project-name": project_name,  # for backward-compatibility
@@ -159,6 +172,7 @@ class Client(TraceDataExtractor):
                     "end_time": _to_iso_format(normalize_datetime(end_time)),
                     "limit": limit,
                     "root_spans_only": root_spans_only,
+                    "orphan_span_as_root_span": orphan_span_as_root_span,
                 },
                 timeout=timeout,
             )
@@ -214,6 +228,7 @@ class Client(TraceDataExtractor):
             return None if df.shape == (0, 0) else df
         return results
 
+    @deprecated("Migrate to using client.spans.get_span_annotations via arize-phoenix-client")
     def get_evaluations(
         self,
         project_name: Optional[str] = None,
@@ -221,6 +236,11 @@ class Client(TraceDataExtractor):
         timeout: Optional[int] = DEFAULT_TIMEOUT_IN_SECONDS,
     ) -> list[Evaluations]:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.spans.get_span_annotations()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Retrieves evaluations for a given project from the Phoenix server or active session.
 
         Args:
@@ -236,7 +256,7 @@ class Client(TraceDataExtractor):
         """
         project_name = project_name or get_env_project_name()
         response = self._client.get(
-            url=urljoin(self._base_url, "v1/evaluations"),
+            url="v1/evaluations",
             params={
                 "project_name": project_name,
                 "project-name": project_name,  # for backward-compatibility
@@ -261,13 +281,14 @@ class Client(TraceDataExtractor):
 
     def _warn_if_phoenix_is_not_running(self) -> None:
         try:
-            self._client.get(urljoin(self._base_url, "arize_phoenix_version")).raise_for_status()
+            self._client.get("arize_phoenix_version").raise_for_status()
         except Exception:
             logger.warning(
-                f"Arize Phoenix is not running on {self._base_url}. Launch Phoenix "
+                f"Arize Phoenix is not running on {self.web_url}. Launch Phoenix "
                 f"with `import phoenix as px; px.launch_app()`"
             )
 
+    @deprecated("Migrate to using client.spans.log_span_annotations via arize-phoenix-client")
     def log_evaluations(
         self,
         *evals: Evaluations,
@@ -275,6 +296,11 @@ class Client(TraceDataExtractor):
         **kwargs: Any,
     ) -> None:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.spans.log_span_annotations()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Logs evaluation data to the Phoenix server.
 
         Args:
@@ -295,14 +321,20 @@ class Client(TraceDataExtractor):
             with pa.ipc.new_stream(sink, table.schema) as writer:
                 writer.write_table(table)
             self._client.post(
-                url=urljoin(self._base_url, "v1/evaluations"),
+                url="v1/evaluations",
                 content=cast(bytes, sink.getvalue().to_pybytes()),
                 headers=headers,
                 timeout=timeout,
             ).raise_for_status()
 
+    @deprecated("Migrate to using client.spans.log_spans via arize-phoenix-client")
     def log_traces(self, trace_dataset: TraceDataset, project_name: Optional[str] = None) -> None:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.spans.log_spans()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Logs traces from a TraceDataset to the Phoenix server.
 
         Args:
@@ -339,7 +371,7 @@ class Client(TraceDataExtractor):
             serialized = otlp_span.SerializeToString()
             content = gzip.compress(serialized)
             response = self._client.post(
-                url=urljoin(self._base_url, "v1/traces"),
+                url="v1/traces",
                 content=content,
                 headers={
                     "content-type": "application/x-protobuf",
@@ -360,7 +392,7 @@ class Client(TraceDataExtractor):
              Dataset: The dataset object.
         """
         response = self._client.get(
-            urljoin(self._base_url, "v1/datasets"),
+            "v1/datasets",
             params={"name": name},
         )
         response.raise_for_status()
@@ -371,6 +403,7 @@ class Client(TraceDataExtractor):
         dataset = records[0]
         return str(dataset["id"])
 
+    @deprecated("Migrate to using client.datasets.get_dataset via arize-phoenix-client")
     def get_dataset(
         self,
         *,
@@ -379,6 +412,11 @@ class Client(TraceDataExtractor):
         version_id: Optional[str] = None,
     ) -> Dataset:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.datasets.get_dataset()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Gets the dataset for a specific version, or gets the latest version of
         the dataset if no version is specified.
 
@@ -402,7 +440,7 @@ class Client(TraceDataExtractor):
             raise ValueError("Dataset id or name must be provided.")
 
         response = self._client.get(
-            urljoin(self._base_url, f"v1/datasets/{quote(id)}/examples"),
+            f"v1/datasets/{quote(id)}/examples",
             params={"version_id": version_id} if version_id else None,
         )
         response.raise_for_status()
@@ -425,6 +463,7 @@ class Client(TraceDataExtractor):
             examples=examples,
         )
 
+    @deprecated("Migrate to using client.datasets.get_dataset_versions via arize-phoenix-client")
     def get_dataset_versions(
         self,
         dataset_id: str,
@@ -432,6 +471,11 @@ class Client(TraceDataExtractor):
         limit: Optional[int] = 100,
     ) -> pd.DataFrame:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.datasets.get_dataset_versions()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Get dataset versions as pandas DataFrame.
 
         Args:
@@ -442,7 +486,7 @@ class Client(TraceDataExtractor):
         Returns:
             pandas DataFrame
         """
-        url = urljoin(self._base_url, f"v1/datasets/{dataset_id}/versions")
+        url = f"v1/datasets/{dataset_id}/versions"
         response = self._client.get(url=url, params={"limit": limit})
         response.raise_for_status()
         if not (records := response.json()["data"]):
@@ -451,6 +495,7 @@ class Client(TraceDataExtractor):
         df["created_at"] = df["created_at"].apply(datetime.fromisoformat)
         return df
 
+    @deprecated("Migrate to using client.datasets.create_dataset via arize-phoenix-client")
     def upload_dataset(
         self,
         *,
@@ -466,6 +511,11 @@ class Client(TraceDataExtractor):
         dataset_description: Optional[str] = None,
     ) -> Dataset:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.datasets.create_dataset()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Upload examples as dataset to the Phoenix server. If `dataframe` or
         `csv_file_path` are provided, must also provide `input_keys` (and
         optionally with `output_keys` or `metadata_keys` or both), which is a
@@ -527,6 +577,7 @@ class Client(TraceDataExtractor):
             dataset_description=dataset_description,
         )
 
+    @deprecated("Migrate to using client.datasets.add_examples_to_dataset via arize-phoenix-client")
     def append_to_dataset(
         self,
         *,
@@ -541,6 +592,11 @@ class Client(TraceDataExtractor):
         metadata: Iterable[Mapping[str, Any]] = (),
     ) -> Dataset:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.datasets.add_examples_to_dataset()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Append examples to dataset on the Phoenix server. If `dataframe` or
         `csv_file_path` are provided, must also provide `input_keys` (and
         optionally with `output_keys` or `metadata_keys` or both), which is a
@@ -601,8 +657,14 @@ class Client(TraceDataExtractor):
             action="append",
         )
 
+    @deprecated("Migrate to using client.experiments.get_experiment via arize-phoenix-client")
     def get_experiment(self, *, experiment_id: str) -> Experiment:
         """
+        .. deprecated::
+            This method is deprecated. Use ``client.experiments.get_experiment()`` via
+            arize-phoenix-client instead.
+            See https://arize-phoenix.readthedocs.io/projects/client/en/latest/
+
         Get an experiment by ID.
 
         Retrieve an Experiment object by ID, enables running `evaluate_experiment` after finishing
@@ -612,7 +674,7 @@ class Client(TraceDataExtractor):
             experiment_id (str): ID of the experiment. This can be found in the UI.
         """
         response = self._client.get(
-            url=urljoin(self._base_url, f"v1/experiments/{experiment_id}"),
+            url=f"v1/experiments/{experiment_id}",
         )
         experiment = response.json()["data"]
         return Experiment.from_dict(experiment)
@@ -676,7 +738,7 @@ class Client(TraceDataExtractor):
             assert_never(table)
         print("📤 Uploading dataset...")
         response = self._client.post(
-            url=urljoin(self._base_url, "v1/datasets/upload"),
+            url="v1/datasets/upload",
             files={"file": file},
             data={
                 "action": action,
@@ -733,7 +795,7 @@ class Client(TraceDataExtractor):
                 )
         print("📤 Uploading dataset...")
         response = self._client.post(
-            url=urljoin(self._base_url, "v1/datasets/upload"),
+            url="v1/datasets/upload",
             headers={"Content-Encoding": "gzip"},
             json={
                 "action": action,
@@ -756,14 +818,14 @@ class Client(TraceDataExtractor):
             raise
         data = response.json()["data"]
         dataset_id = data["dataset_id"]
-        response = self._client.get(
-            url=urljoin(self._base_url, f"v1/datasets/{dataset_id}/examples")
-        )
+        path = f"v1/datasets/{dataset_id}/examples"
+        response = self._client.get(path)
         response.raise_for_status()
         data = response.json()["data"]
         version_id = data["version_id"]
         examples = data["examples"]
-        print(f"💾 Examples uploaded: {self.web_url}datasets/{dataset_id}/examples")
+        examples_url = urljoin(self.web_url, f"datasets/{dataset_id}/examples")
+        print(f"💾 Examples uploaded: {examples_url}")
         print(f"🗄️ Dataset version ID: {version_id}")
 
         return Dataset(
